<link rel="stylesheet" href="https://cdn.jsdelivr.net/npm/hint.css@2.7.0/hint.min.css">
<link rel="stylesheet" href="https://cdnjs.cloudflare.com/ajax/libs/highlight.js/11.6.0/styles/{{ site.highlightjs-style|default: "default" }}.min.css">
<link rel="stylesheet" href="{{ "/assets/css/code.css" | relative_url }}">
<script defer src="https://cdnjs.cloudflare.com/ajax/libs/highlight.js/11.6.0/highlight.min.js"></script>
{% include highlight_additional.html %}
<script defer src="{{ "/assets/js/code.js" | relative_url }}"></script>